require 'chronic'
require 'json'
require 'csv'
require 'time'
require 'thor'

require 'qntf/storage/json'

class Qntf::CLI < Thor
  desc "list", "list available datasets"

  def list
    datasets = Qntf::Json::Dataset.all

    datasets.each do |d|
      puts d.name
    end
  end

  desc 'data [NAME]', 'get the data of a dataset'
  def data(name)
    dataset = Qntf::Json::Dataset.new(name)
    dataset.data.each do |d|
      puts d
    end
  end

  desc "new", "create a new dataset"
  def new
    name = ask('Name:').chomp
    fields = ask('Fields(seperated by comma):').split(',').map(&:strip)
    puts "Creating dataset #{name}"
    Qntf::Json::Dataset.new(name, fields)
  end

  desc "add_field [NAME]", "add a field to a dataset"
  def add_field(name)
    field = ask('Field name:').chomp
    dataset = Qntf::Json::Dataset.new(name)
    dataset.fields.push(field)
  end

  desc "fields [NAME]", "get the fields of a dataset"
  def fields(name)
    dataset = Qntf::Json::Dataset.new(name)
    puts dataset.fields.join(', ')
  end

  desc "store [NAME]", "store data in dataset"
  def store(name)
    dataset = Qntf::Json::Dataset.new(name)
    hash = {}
    for field in dataset.fields
      value = ask("#{field}:").chomp
      hash[field] = parseNum(value) unless value.empty?
    end

    date = ask_time

    dataset.add_datapoint(hash, date)
  end

  private
  def parseNum(v)
    v.match('\.').nil? ? Integer(v) : Float(v) rescue v.to_s
  end

  def ask_time
    loop do
      string = ask("time:").chomp
      if string.empty?
        return nil
      end
      time = Chronic.parse(string)

      if ask("#{time.to_s}, continue? [y/n]").chomp == 'y'
        return time.utc.iso8601(6)
      end
    end
  end
end
