require 'json'
require 'fileutils'
require 'time'

module Qntf
  module Json
    class Dataset
      def self.all
        files = Dir.glob(source + '/*.json')
        files.map { |x| new(File.basename(x, '.json')) }
      end

      def self.source
        ENV['QNTFDIR'] || "#{ENV['HOME']}/.qntf"
      end

      attr_accessor :name, :path, :fields, :data

      def initialize(name, fields = [])
        @name = name
        @fields = fields
        @data = {}

        @path = File.expand_path("#{name}.json", self.class.source)

        if File.exist?(@path) and !File.zero?(@path)
          load
        else
          bootstrap
        end
      end

      def add_field(field)
        @fields << field
        save
      end

      def add_datapoint(datapoint, date=nil)
        date ||= Time.now.utc.iso8601(6)
        data[date] = datapoint
        save
      end

      def to_hash
        {fields: @fields, data: @data}
      end

      def load
        data = JSON.parse(File.read(@path))
        @fields = data['fields']
        @data =   data['data']
      end

      def save
        File.open(@path, 'w') do |file|
          file.puts to_hash.to_json
        end
      end

      private

      def bootstrap
        FileUtils.mkdir(self.class.source) unless File.directory?(self.class.source)
        FileUtils.touch(@path)
        save
      end
    end
  end
end
